/***************************************************************************
                          keppropagator.h  -  description
                             -------------------
    begin                : Sat Jan 13 2001
    copyright            : (C) 2001 by Edson Pereira
    email                : n1vtn@arrl.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
//
// propagator.h
//

#ifndef KEPPROPAGATOR_H
# define KEPPROPAGATOR_H

class KepPropagator
{

public:
   KepPropagator();
   ~KepPropagator();

   int init(char* name, char* line1, char* line2,
	    double _stnlat, double _stnlong, int _srnalt);
   long DayNum(int m, int d, int y);
   double CurrentDaynum();
   char *daynum2String(double daynum);
   int daynum2unix(double daynum);
   long daynum2time_t( double daynum );
   char AosHappens();
   char Decayed();
   char Geostationary();
   void propagate( double _daynum );
   void propagate( long t );
   char *getSatName();
   long getCatalogNumber();
   double getDaynum();
   double getSspLat();
   double getSspLon();
   double getAzimuth();
   double getElevation();
   double getAltitude();
   double getSlantRange();
   double getVelocity();
   double getFootprint();
   double getPhase();
   long getOrbitNumber();
   double getDoppler( double f );
   double getPathLoss( double f );
   double getAOSTime();
   double getLOSTime();
   char *getTLELine1();
   char *getTLELine2();
   
private:

   char name[25];
   long catnum;
   int year;
   double refepoch;
   double incl;
   double raan;
   double eccn;
   double argper;
   double meanan;
   double meanmo;
   double drag;
   long orbitnum;

   char callsign[10];
   double stnlat;
   double stnlong;
   int stnalt;

   char    qthfile[50], tlefile[50], temp[80], output[20];

   unsigned char val[256];

   int     indx, ma256, iaz, iel, isplat, isplong;

   long    i1, i2, i3, de, rv, irk;

   double  age, epoch, sma, range, t1, se, e1, e2, n0, c[4][3],
       k2, s1, c1, l8, s9, c9, s8, c8, r9, z9, x9, y9, o, w, q, s0, s2,
       c2, q0, m, e,  s3, c3, c0, r3, m1, m5, x0, yzero, x1, yone, r,
       z1, g7, s7, c7, x,  y,  z, x5, y5, z5, z8, x8, y8, df, aostime,
       lostime, apogee, perigee, azimuth, ssplat, ssplong, elevation,
       vk, vm, rm, rk, ak, am, fk, fm, yr;

   double Daynum, SSPlat, SSPlon, Azimuth, Elevation, Altitude, 
          SlantRange, Velocity, FootPrint, Phase;
   int OrbitNumber, Catnum;
   
   double TP, PT, S1, C1, deg2rad, R0, FF, KM;

   char TLEline1[80];
   char TLEline2[80];

   double old_clock, old_range, dp, dt, doppler;

   double FixAngle(double x);
   char *SubString(char *string, char start, char end);
   void CopyString(unsigned char *source, unsigned char *destination,
		   unsigned char start, unsigned char end);
   char KepCheck(char *line1, char *line2);
   double PreCalc(double daynum);
   void Calc();
};

#endif
