/* Copyright (C) 2002 Edson Pereira, PU1JTE, N1VTN, 7N4NCL
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, download a copy
 * from http://www.fsf.org
 *
 */

#include "petittrack.h"
#include "world0.xpm"
#include "world1.xpm"

PetitTrack::PetitTrack( QWidget* parent,  const char* name, WFlags fl )
   : QWidget( parent, name, fl )
{
   if ( !name ) setName( "PetitTrack" );
   resize( 196, 245 );
   setCaption( tr( "PetitTrack 1.1.1" ) );
   layout = new QVBoxLayout( this );
   layout->setSpacing( 1 );
   layout->setMargin( 0 );

   MyConfiguration = new Config(tr( "PetitTrack" ));
   MyConfiguration->setGroup( "petittrack group" );

   lat = atof( MyConfiguration->readEntry("lat", "0.0" ).latin1() );
   lon = atof( MyConfiguration->readEntry("lon", "0.0").latin1() );
   alt = atoi( MyConfiguration->readEntry("alt", "10").latin1() );
   mapType = atoi( MyConfiguration->readEntry( "map", "0").latin1() );
   rt1ObjectID = atoi( MyConfiguration->readEntry("obj", "0").latin1() );

   char *home = getenv( "HOME" );
   snprintf( tleFile, sizeof(tleFile), "%s/Documents/nasatle.txt", home );

   QFont font( "fixed", 12 );
   font.setFixedPitch( true );

   // Setup the Real Time View
   rtnView = new QListView( this, "textView" );
   rtnView->setFont( font );
   rtnView->addColumn( "Object", 68 );
   rtnView->addColumn( " ", 8 );
   rtnView->addColumn( "AOS", 68 );
   rtnView->addColumn( "LOS", 68 );
   rtnView->setSorting( 2 );
   connect( rtnView, SIGNAL(pressed(QListViewItem *)),
	    this, SLOT(itemSelected( QListViewItem *)));

   // Setup the Single View
   rt1Frame = new QFrame( this );
   rt1ButtonGroup = new QButtonGroup( rt1Frame );
   rt1ButtonGroup->setExclusive( true );
   rt1ButtonGroup->setGeometry( 184, 0, 55, 264 );
   rt1ButtonGroup->setFrameShape( QFrame::NoFrame );
   connect( rt1ButtonGroup, SIGNAL(clicked(int)), this, SLOT(setRT1Object(int)) );

   QFrame *rt1LabelFrame = new QFrame( rt1Frame );
   rt1LabelFrame->setGeometry( 0, 121, 232, 133 );
   rt1LabelFrame->setFrameStyle( QFrame::Box | QFrame::Plain );

   QLabel *ll[14];

   for ( int i = 0; i < 10; i++ )
   {
      ll[i] = new QLabel( rt1LabelFrame );
      ll[i]->setFont( font );
      ll[i]->setGeometry( 1, 1 + ( 13 * i ), 70, 13 );

      rtLabel[i] = new QLabel( rt1LabelFrame );
      rtLabel[i]->setFont( font );
      rtLabel[i]->setGeometry( 70, 1+ ( 13 * i ), 161, 13 );
      if ( i % 2 == 0 ) rtLabel[i]->setBackgroundColor( QColor( 255,255,255 ) );
      else rtLabel[i]->setBackgroundColor( QColor( 220,220,255 ) );
   }

   aoslosLabel = ll[2];

   ll[0]->setText( "    Object: " );
   ll[1]->setText( " Date/Time: " );
   ll[2]->setText( "       AOS: " );
   ll[3]->setText( " CountDown: " );
   ll[4]->setText( "   Lat/Lon: " );
   ll[5]->setText( "     Az/El: " );
   ll[6]->setText( " Alt/Range: " );
   ll[7]->setText( " Phase/Rev: " );
   ll[8]->setText( "  PathLoss: " );
   ll[9]->setText( "   Doppler: " );

   map0 = new QPixmap( (const char **)world0_xpm );
   map1 = new QPixmap( (const char **)world1_xpm );
   worldPix = map0;
   
   QFrame *worldFrame = new QFrame( rt1Frame );
   worldFrame->setGeometry( 0, 2, worldPix->width() + 2, worldPix->height() + 2 );
   worldFrame->setFrameStyle( QFrame::Box | QFrame::Plain );

   worldMap = new QFrame( worldFrame );
   worldMap->setGeometry( 1, 1, worldPix->width(), worldPix->height() );
   //worldMap->setBackgroundPixmap( *worldPix );

   initLabel = new QLabel( rt1Frame );
   initLabel->setGeometry( 0, 0, 232, 256 );
   initLabel->setAlignment( AlignHCenter );
   initLabel->setText( QString("\n\n\n\nPetitTrack\n\n\n") +
		       "Copyright (C) 2002 Edson Pereira\n" +
		       "\n\n\n\n\nInitializing...\n" );
   ephemFrame = new QFrame( this );

   QPushButton *ephemButton = new QPushButton( ephemFrame );
   ephemButton->setText( "Calculate" );
   ephemButton->setGeometry( 80, 237, 74, 20 );
   connect( ephemButton, SIGNAL(clicked()), this, SLOT(computeEphem()) );

   QPushButton *ephemStop = new QPushButton( ephemFrame );
   ephemStop->setText( "Stop" );
   ephemStop->setGeometry( 156, 237, 74, 20 );
   connect( ephemStop, SIGNAL(clicked()), this, SLOT(stopEphem()) );

   ephemStopFlag = false;
   
   QLabel *ephemDaysLabel = new QLabel( "Days", ephemFrame );
   ephemDaysLabel->setGeometry( 2, 237, 30, 20 );
   
   ephemDays = 7;
   ephemDaysSpin = new QSpinBox( ephemFrame, "ephemDaysSpin" );
   ephemDaysSpin->setGeometry( 33, 237, 45, 20 );
   ephemDaysSpin->setMinValue( 2 );
   ephemDaysSpin->setValue( ephemDays );
   connect( ephemDaysSpin, SIGNAL(valueChanged(int)), this, SLOT(setEphemDays(int)));
   
   ephemView = new QMultiLineEdit( ephemFrame );
   ephemView->setGeometry( 0, 2, 230, 235 );
   ephemView->setFont( font );
   ephemView->setReadOnly( true );

   QFrame *astroFrame = new QFrame( this );
   
   QLabel *sunLabel = new QLabel( "--- Sun ---", astroFrame );
   sunLabel->setFont( font );
   sunLabel->setGeometry( 2, 2, 230, 15 );
   sunLabel->setBackgroundColor( QColor( 255, 255, 220 ) );
   sunLabel->setFrameStyle( QFrame::Box | QFrame::Plain );
   
   QFrame *sunFrame = new QFrame( astroFrame );
   sunFrame->setGeometry( 2, 18, 230, 62 );
   sunFrame->setFrameStyle( QFrame::Box | QFrame::Plain );
   
   QLabel *sunLabel2 = new QLabel( " Elevation:", sunFrame );
   sunLabel2->setFont( font );
   sunLabel2->setGeometry( 4, 1, 70, 15 );
   
   sunEleLabel = new QLabel( sunFrame );
   sunEleLabel->setFont( font );
   sunEleLabel->setGeometry( 72, 1, 157, 15 );
   sunEleLabel->setBackgroundColor( QColor( 255, 255, 255 ) );
   
   QLabel *sunLabel1 = new QLabel( "   Azimuth:", sunFrame );
   sunLabel1->setFont( font );
   sunLabel1->setGeometry( 4, 16, 70, 15 );
   
   sunAziLabel = new QLabel( sunFrame );
   sunAziLabel->setFont( font );
   sunAziLabel->setGeometry( 72, 16, 157, 15 );
   sunAziLabel->setBackgroundColor( QColor( 230, 255, 230 ) );
   
   QLabel *sunLabel3 = new QLabel( "  Latitude:", sunFrame );
   sunLabel3->setFont( font );
   sunLabel3->setGeometry( 4, 31, 70, 15 );
   
   sunLatLabel = new QLabel( sunFrame );
   sunLatLabel->setFont( font );
   sunLatLabel->setGeometry( 72, 31, 157, 15 );
   sunLatLabel->setBackgroundColor( QColor( 255, 255, 255 ) );
   
   QLabel *sunLabel4 = new QLabel( " Longitude:", sunFrame );
   sunLabel4->setFont( font );
   sunLabel4->setGeometry( 4, 46, 70, 15 );
   
   sunLonLabel = new QLabel( sunFrame );
   sunLonLabel->setFont( font );
   sunLonLabel->setGeometry( 72, 46, 157, 15 );
   sunLonLabel->setBackgroundColor( QColor( 230, 255, 230 ) );
   
   QLabel *moonLabel = new QLabel( "-- Moon ---", astroFrame );
   moonLabel->setFont( font );
   moonLabel->setGeometry( 2, 83, 230, 15 );
   moonLabel->setBackgroundColor( QColor( 200, 200, 200 ) );
   moonLabel->setFrameStyle( QFrame::Box | QFrame::Plain );
   
   QFrame *moonFrame = new QFrame( astroFrame );
   moonFrame->setGeometry( 2, 99, 230, 62 );
   moonFrame->setFrameStyle( QFrame::Box | QFrame::Plain );
   
   QLabel *moonLabel2 = new QLabel( " Elevation:", moonFrame );
   moonLabel2->setFont( font );
   moonLabel2->setGeometry( 4, 1, 70, 15 );
   
   moonEleLabel = new QLabel( moonFrame );
   moonEleLabel->setFont( font );
   moonEleLabel->setGeometry( 72, 1, 157, 15 );
   moonEleLabel->setBackgroundColor( QColor( 255, 255, 255 ) );
   
   QLabel *moonLabel1 = new QLabel( "   Azimuth:", moonFrame );
   moonLabel1->setFont( font );
   moonLabel1->setGeometry( 4, 16, 70, 15 );
   
   moonAziLabel = new QLabel( moonFrame );
   moonAziLabel->setFont( font );
   moonAziLabel->setGeometry( 72, 16, 157, 15 );
   moonAziLabel->setBackgroundColor( QColor( 230, 255, 230 ) );
   
   QLabel *moonLabel3 = new QLabel( "  Latitude:", moonFrame );
   moonLabel3->setFont( font );
   moonLabel3->setGeometry( 4, 31, 70, 15 );
   
   moonLatLabel = new QLabel( moonFrame );
   moonLatLabel->setFont( font );
   moonLatLabel->setGeometry( 72, 31, 157, 15 );
   moonLatLabel->setBackgroundColor( QColor( 255, 255, 255 ) );
   
   QLabel *moonLabel4 = new QLabel( " Longitude:", moonFrame );
   moonLabel4->setFont( font );
   moonLabel4->setGeometry( 4, 46, 70, 15 );
   
   moonLonLabel = new QLabel( moonFrame );
   moonLonLabel->setFont( font );
   moonLonLabel->setGeometry( 72, 46, 157, 15 );
   moonLonLabel->setBackgroundColor( QColor( 230, 255, 230 ) );
   
   QFrame *configFrame = new QFrame( this );
   QLabel *lonLabel = new QLabel( "Observer's Longitude", configFrame );
   QLabel *latLabel = new QLabel( "Observer's Latitude", configFrame );
   QLabel *altLabel = new QLabel( "Observer's Altitude", configFrame );

   lonLine = new QLineEdit( configFrame );
   latLine = new QLineEdit( configFrame );
   altLine = new QLineEdit( configFrame );

   lonLabel->setGeometry( 2, 2, 100, 17 );
   latLabel->setGeometry( 2, 22, 100, 17 );
   altLabel->setGeometry( 2, 42, 100, 17 );

   lonLine->setGeometry( 104, 2, 128, 17 );
   latLine->setGeometry( 104, 22, 128, 17 );
   altLine->setGeometry( 104, 42, 128, 17 );

   latLine->setText( MyConfiguration->readEntry("lat", "0.0" ).latin1() );
   lonLine->setText( MyConfiguration->readEntry("lon", "0.0" ).latin1() );
   altLine->setText( MyConfiguration->readEntry("alt", "0" ).latin1() );
   
   mapButtonGroup = new QButtonGroup( configFrame );
   mapButtonGroup->setTitle( "Map Type" );
   mapButtonGroup->setExclusive( true );
   
   QRadioButton *mapButton0 = new QRadioButton( "Full Color Map", mapButtonGroup );
   mapButton0->setGeometry( 5, 15, 100, 15 );
   QRadioButton *mapButton1 = new QRadioButton( "Two Color Map", mapButtonGroup );
   mapButton1->setGeometry( 5, 33, 100, 15 );

   mapButtonGroup->insert( mapButton0, 0 );
   mapButtonGroup->insert( mapButton1, 1 );
   mapButtonGroup->setGeometry( 2, 62, 230, 53 );
   connect( mapButtonGroup, SIGNAL(clicked(int)), this, SLOT(setMapType(int)));
   mapButtonGroup->setButton( mapType );
   setMapType( mapType );
   
   QFrame *aboutFrame = new QFrame( this );
   QLabel *aboutLabel = new QLabel( aboutFrame );
   aboutLabel->setGeometry( 20, 20, 200, 300 );
   aboutLabel->setText( QString( "PetitTrack v1.1.1\n\n" ) +
			"Copyright (c) 2002\n" +
			"Edson Pereira, PU1JTE, N1VTN, 7N4NCL\n\n" +
			"This program is free software; you can" + 
			"\nredistribute it and/or modify it under" +
			"\nthe terms of the GNU General Public " + 
			"\nLicense Version 2 as published by the" + 
			"\nFree Software Foundation." +
			"\n\nCredits: \n\n" +
			"Keplerian Propagator algorithm from\n" +
			"John A. Magliacane, KD2BD's Predict.\n\n\n\n\n\n" );

   tab = new QTabWidget( this, "tab" );
   tab->addTab( rt1Frame, "Tracker" );
   tab->addTab( rtnView, "Queue" );
   tab->addTab( ephemFrame, "Ephemeris" );
   tab->addTab( astroFrame, "Astro" );
   tab->addTab( configFrame, "Config" );
   tab->addTab( aboutFrame, "About" );

   layout->addWidget( tab );

   for ( int i = 0; i < MAXOBJECTS; i++ )
   {
      aosTime[i] = 0;
      losTime[i] = 0;
   }
   
   loadTLE();

   groundTrackFlag = false;
   groundTrackPoints = new QPointArray( 60 );

   t1 = new QTimer( this );
   connect( t1, SIGNAL(timeout()), this, SLOT(propagate()));
   t1->start( 1000, false );

   ephemTimer = new QTimer( this );
   connect( ephemTimer, SIGNAL(timeout()), this, SLOT(runEphem()));
   calculating = false;
   
   calculateGroundTrack();
   
   moon = new MoonPropagator( lat, lon );
   sun = new SunPropagator( lat, lon );   
   sunPoints = new QPointArray( 360 );
   calculateSunFootprint();
}

PetitTrack::~PetitTrack()
{
   char tmp[3];
   bool ok;

   sprintf( tmp, "%d", rt1ObjectID );
   MyConfiguration->writeEntry( "obj", tmp );

   sprintf( tmp, "%d", mapType );
   MyConfiguration->writeEntry( "map", tmp );

   latLine->text().toDouble( &ok );
   if ( ok ) MyConfiguration->writeEntry( "lat", latLine->text() );

   lonLine->text().toDouble( &ok );
   if ( ok ) MyConfiguration->writeEntry( "lon", lonLine->text() );

   altLine->text().toInt( &ok );
   if ( ok ) MyConfiguration->writeEntry( "alt", altLine->text() );

   delete MyConfiguration;
}

void PetitTrack::loadTLE()
{
   FILE *fp;
   char line0[ 81 ];
   char line1[ 81 ];
   char line2[ 81 ];
   char tmpname[ 81 ];
   char fragname[ 81 ];
   char index[ 3 ];
   int linenum = 0;
   unsigned int i;
   char *r;
   char temp[256];

   numObjects = 0;

   fp = fopen( tleFile, "r" );
   if ( fp == NULL )
   {
      perror( tleFile );
      QMessageBox::critical( 0, "PetitTrack",
			     QString("PetitTrack could  not find the file\n") + tleFile +
			     "\n\nPlease download a recent" +
			     "\nNASA TLE file from " +
			     "\nhttp://www.celestrak.com or" +
			     "\nany other source and place" +
			     "\nthe file in the directory" + 
			     "\n" + getenv("HOME") + "/Documents." +
			     "\n\nThe file must have be named" +
			     "\nnasatle.txt" +
			     "\n\nPetitTrack will now exit." );
      exit(1);
   }

   while( numObjects < MAXOBJECTS )
   {
      r = fgets( line0, 80, fp );
      if ( r == NULL ) break;
      if ( line0[0] == '\n' ) continue;
      linenum++;

      // Clean up the trailing spaces 
      for ( i = 0; i < strlen(line0); i++ )
      {
	 if ( line0[i] == '\r' || line0[i] == '\n' ) 
	    line0[i] = '\0';
      }
      for ( i = strlen( line0 ) - 1; i > 0; i-- )
      {
	 if ( line0[i] != ' ' ) 
	 {
	    line0[i+1] = '\0';
	    break;
	 }
      }
      
      // If name is longer than 11 characters, shrink name
      if ( strlen( line0 ) > 11 )
      {
	 strcpy( fragname, line0 );
	 fragname[5] = '\0';	    
	 sprintf( tmpname, "%s~%s", fragname, line0 + strlen(line0) - 5 );
	 strcpy( line0, tmpname );
      }
      
      fgets( line1, 80, fp );
      if ( line1[0] != '1' )
      {
	 sprintf( temp, "Unexpected error at TLE file line %d\n", linenum );
	 QMessageBox::critical( 0, "PetitTrack",
				QString( temp ) + "\n\n" +
				"TLE File: \n" + tleFile +
				"\n\nPlease check the TLE File integrity" +
				"\n\nPetitTrack will now exit." );
	 exit(1);
      }
      linenum++;

      fgets( line2, 80, fp );
      if ( line2[0] != '2' )
      {
	 sprintf( temp, "Unexpected error at TLE file line %d\n", linenum );
	 QMessageBox::critical( 0, "PetitTrack",
				QString( temp ) + "\n\n" +
				"TLE File: \n" + tleFile +
				"\n\nPlease check the TLE File integrity" +
				"\n\nPetitTrack will now exit." );
	 exit(1);
      }
      linenum++;

      p[ numObjects ] = new KepPropagator();
      p[ numObjects ]->init( line0, line1, line2, lat, lon, alt );

      sprintf( index, "%d", numObjects );
      listViewItem[ numObjects ] = new QListViewItem( rtnView, line0, " ", "?", "?", QString( index ) );

      numObjects++;
   }
   
   if ( rt1ObjectID > numObjects ) rt1ObjectID = 0;
   rtnView->setCurrentItem( listViewItem[ rt1ObjectID ] );
}

void PetitTrack::findAOSLOS( double daynum, KepPropagator *kp, int satIndex )
{
   bool inview = false;

   if ( !kp->AosHappens() )
   {
      aosTime[satIndex] = -1.0;
      losTime[satIndex] = -1.0;
      return;
   }
   
   for ( ;; )
   {
      kp->propagate( daynum );

      if ( !inview && kp->getElevation() >= 0.0 )
      {
	 inview = true;
	 aosTime[ satIndex ] = daynum;
      }
      if ( inview && kp->getElevation() < 0.0 )
      {
	 inview = false;
	 losTime[ satIndex ] = daynum;
	 break;
      }
      daynum = incrementDaynum( daynum, kp->getElevation() );
   }
}

void PetitTrack::propagate()
{
   static int i = rt1ObjectID;
   static int init = true;
   double daynum;
   struct timeb tptr;
   char temp[256];
   struct tm *aos_tm;
   struct tm *los_tm;
   struct tm *temp_tm;
   time_t t;

   ftime( &tptr );

   daynum = ((((double)tptr.time + 0.001 *
	       (double)tptr.millitm) / 86400.0 ) - 3651.0);

   if ( tab->currentPageIndex() == 3 )
   {
      sun->propagate( daynum );
      sprintf( temp, " %7.2f%c %s", sun->getEl(), 0xb0,
	       sun->getEl() > 0.0 ? "*" : " " );
      sunEleLabel->setText( temp );
      sprintf( temp, " %7.2f%c", sun->getAz(), 0xb0 );
      sunAziLabel->setText( temp );
      sprintf( temp, " %7.2f%c", sun->getLat(), 0xb0 );
      sunLatLabel->setText( temp );
      sprintf( temp, " %7.2f%c", sun->getLon(), 0xb0 );
      sunLonLabel->setText( temp );
   
      moon->propagate( daynum );
      sprintf( temp, " %7.2f%c %s", moon->getEl(), 0xb0,
	       moon->getEl() > 0.0 ? "*" : " " );
      moonEleLabel->setText( temp );
      sprintf( temp, " %7.2f%c", moon->getAz(), 0xb0 );
      moonAziLabel->setText( temp );
      sprintf( temp, " %7.2f%c", moon->getLat(), 0xb0 );
      moonLatLabel->setText( temp );
      sprintf( temp, " %7.2f%c", moon->getLon(), 0xb0 );
      moonLonLabel->setText( temp );
   }
   if ( tptr.time % 120 == 0 )
   {
      calculateGroundTrack();
      calculateSunFootprint();
   }
   
   if ( aosTime[i] < 0.0 )
   {
      sprintf( temp, "Never" );
      listViewItem[i]->setText( 2, temp );
      listViewItem[i]->setText( 3, temp );
   }
   else if ( losTime[i] <= daynum )
   {
      findAOSLOS( daynum, p[i], i );

      if ( aosTime[i] > 0.0 )
      {
	 t = p[i]->daynum2unix( aosTime[i] );
	 aos_tm = gmtime( &t );
	 strftime( temp, 20, "%m%d %R",  aos_tm );
	 listViewItem[i]->setText( 2, temp );

	 t = p[i]->daynum2unix( losTime[i] );
	 los_tm = gmtime( &t );
	 strftime( temp, 20, "%m%d %R",  los_tm );
	 listViewItem[i]->setText( 3, temp );
      }
      rtnView->sort();
   }

   if ( daynum >= aosTime[i] && aosTime[i] > 0.0 ) sprintf( temp, "*" );
   else sprintf( temp, " " );
   listViewItem[i]->setText( 1, temp );

   p[rt1ObjectID]->propagate( daynum );

   if ( p[rt1ObjectID]->getElevation() >= 0.0 )
   {
      sprintf( temp, "  %-11s <<>>", p[rt1ObjectID]->getSatName() );
      rtLabel[0]->setBackgroundColor( QColor( 220,255,220 ) );
   }
   else
   {
      sprintf( temp, "  %-9s", p[rt1ObjectID]->getSatName() );
      rtLabel[0]->setBackgroundColor( QColor( 255,255,255 ) );
   }
   rtLabel[0]->setText( temp );

   t = p[rt1ObjectID]->daynum2unix( daynum );
   temp_tm = gmtime( &t );
   strftime( temp, 40, "  %m%d      %T  UTC",  temp_tm );
   rtLabel[1]->setText( temp );

   if ( p[rt1ObjectID]->getElevation() >= 0.0 )
   {
      t = p[rt1ObjectID]->daynum2unix( losTime[rt1ObjectID] );
      los_tm = gmtime( &t );
      strftime( temp, 40, "  %m%d      %T  UTC",  los_tm );
      rtLabel[2]->setText( temp );
      sprintf( temp, "       LOS: " );
      aoslosLabel->setText( temp );
   }
   else
   {
      t = p[rt1ObjectID]->daynum2unix( aosTime[rt1ObjectID] );
      aos_tm = gmtime( &t );
      strftime( temp, 40, "  %m%d      %T  UTC",  aos_tm );
      rtLabel[2]->setText( temp );
      sprintf( temp, "       AOS: " );
      aoslosLabel->setText( temp );
   }

   double countDownTime1;
   if ( p[rt1ObjectID]->getElevation() < 0 ) 
      countDownTime1 = aosTime[rt1ObjectID] - daynum + 1.0;
   else 
      countDownTime1 = losTime[rt1ObjectID] - daynum + 1.0;
   t = p[rt1ObjectID]->daynum2unix( countDownTime1 );
   temp_tm = gmtime( &t );
   sprintf( temp, "    %2d      %02d:%02d:%02d",
	    temp_tm->tm_mday - 1,
	    temp_tm->tm_hour,
	    temp_tm->tm_min,
	    temp_tm->tm_sec );

   rtLabel[3]->setText( temp );

   sprintf( temp, " %5.1f%c     %5.1f%c",
	    p[rt1ObjectID]->getSspLat(), 0xb0,
	    p[rt1ObjectID]->getSspLon(), 0xb0  );
   rtLabel[4]->setText( temp );

   sprintf( temp, " %5.1f%c     %5.1f%c",
	    p[rt1ObjectID]->getAzimuth(), 0xb0,
	    p[rt1ObjectID]->getElevation(), 0xb0);
   rtLabel[5]->setText( temp );

   sprintf( temp, " %5.0f km   %5.0f km",
	    p[rt1ObjectID]->getAltitude(),
	    p[rt1ObjectID]->getSlantRange());
   rtLabel[6]->setText( temp );
   
   sprintf( temp, "   %3.0f     %6ld",
	    p[rt1ObjectID]->getPhase() * 256,
	    p[rt1ObjectID]->getOrbitNumber());
   rtLabel[7]->setText( temp );

   sprintf( temp, " %3.1f dB   %3.1f dB", 
	    p[rt1ObjectID]->getPathLoss( 435.0e6 ),
	    p[rt1ObjectID]->getPathLoss( 2401.0e6 ));
   rtLabel[8]->setText( temp );

   sprintf( temp, " %5.0f Hz  %6.0f Hz",  
	    p[rt1ObjectID]->getDoppler( 435.0e6 ),
	    p[rt1ObjectID]->getDoppler( 2401.0e6 ));
   rtLabel[9]->setText( temp );

   drawMap( p[rt1ObjectID] );

   i = (i + 1) % numObjects;
   
   if ( init )
   {
      initLabel->hide();
      init = false;
   }
}

void PetitTrack::drawMap( KepPropagator *kp )
{
   QPen pen1, pen2, pen3, pen4, pen5, pen6;
   QPointArray pa( 360 );
   int x, y, xx, yy;

   // Fragments of the code below derived from KD2DB's Predict
   double spiderlat, spiderlong, azimuth, ssplat, ssplong, TWOPI,
      deg2rad = 1.74532925199e-02, R0=6378.16,
      beta, num, dem;

   int azi;

   float height = (float)worldPix->height();
   float width = (float)worldPix->width();

   TWOPI = 2.0 * M_PI;

   ssplat = kp->getSspLat() * deg2rad;
   ssplong = kp->getSspLon() * deg2rad;
   beta = 0.5 * kp->getFootprint() / R0;

   /* Compute the visibility circle */
   for ( azi = 0; azi < 360; azi++)
   {
      azimuth = deg2rad * (double)azi;
      spiderlat = asin( sin( ssplat ) * cos( beta )
			+ cos( azimuth ) * sin( beta ) * cos( ssplat ));
      num = cos( beta ) - ( sin( ssplat ) * sin( spiderlat ));
      dem = cos( ssplat ) * cos( spiderlat );

      if ( azi == 0 && ( beta > M_PI_2 - ssplat ))
	 spiderlong = ssplong + M_PI;
      else if ( azi == 180 && ( beta > M_PI_2 + ssplat ))
	 spiderlong = ssplong + M_PI;
      else if ( fabs( num / dem) > 1.0 )
	 spiderlong = ssplong;
      else
      {
	 if (( 180 - azi ) >= 0)
	    spiderlong = ssplong - arccos( num, dem );
	 else
	    spiderlong = ssplong + arccos( num, dem );
      }

      while ( spiderlong < 0.0 )
	 spiderlong += TWOPI;

      while ( spiderlong > ( 2.0 * M_PI ))
	 spiderlong -= TWOPI;

      spiderlat = spiderlat / deg2rad;
      spiderlong = spiderlong / deg2rad;

      /* Convert spiderweb data to map-based coordinates */
      if ( spiderlong <= 180.0 )
	 xx = (int)( height - ( width * ( spiderlong / 360.0 )));
      else
      {
	 spiderlong = spiderlong - 180.0;
	 xx = (int)( width - ( width * ( spiderlong / 360.0 )));
      }

      yy = (int)( height / 2.0 - ( height / 2.0 * ( spiderlat / 90.0 )));
      pa.setPoint( azi, xx, yy );
   }

   // The sub satellite point map coordinates 
   if ( kp->getSspLon() <= 180.0) x = (int)( height - (width * ( kp->getSspLon() / 360.0 )));
   else x = (int)( width - ( width * ( (kp->getSspLon() - 180.0) / 360.0 )));

   y = (int)( height / 2.0 - ( height / 2.0 * ( kp->getSspLat() / 90.0 )));

   // The observer map coordinates
   int lonx, laty;
   if ( lon < 0.0 ) lonx = (int)( width - ( width * (( 180.0 - fabs(lon)) / 360.0 )));
   else lonx = (int)(  ( width * (( 180.0 - lon )/ 360.0 )));

   laty = (int)( height / 2.0 - ( height / 2.0 * ( lat / 90.0 )));

   QPainter p;
   QPixmap tmpMap = *worldPix;
   p.begin( &tmpMap );

   pen1.setWidth( 1 );
   pen2.setWidth( 1 );
   pen3.setWidth( 1 );
   pen4.setWidth( 1 );
   pen5.setWidth( 1 );
   pen6.setWidth( 1 );

   pen1.setColor( QColor( 255, 0, 0 ) );
   pen2.setColor( QColor( 0, 200, 255 ) );
   pen3.setColor( QColor( 255, 255, 0 ) );
   pen4.setColor( QColor( 0, 255, 0 ) );
   pen5.setColor( QColor( 255, 255, 255 ) );
   pen6.setColor( QColor( 150, 150, 150 ) );
   
   // Draw the satellite trajectory
   p.setPen( pen4 );
   p.drawPoints( *groundTrackPoints );

   // Draw the sun terminator
   p.setPen( pen6 );
   p.drawPoints( *sunPoints );

   // Draw the satellite footprint
   p.setPen( pen1 );
   p.drawRect( x - 1, y - 1, 3, 3 );
   p.drawPoints( pa );

   // Draw the observer
   p.setPen( pen2 );
   p.drawRect( lonx - 1, laty - 1, 3, 3 );
   
   // Draw the moon
   p.setPen( pen5 );
   p.drawLine( moon_x - 1, moon_y, moon_x + 1, moon_y );
   p.drawLine( moon_x, moon_y - 1, moon_x, moon_y + 1 );
   
   // Draw the sun 
   p.setPen( pen1 );
   p.drawLine( sun_x - 1, sun_y, sun_x + 1, sun_y );
   p.drawLine( sun_x, sun_y - 1, sun_x, sun_y + 1 );
   p.setPen( pen3 );
   p.drawPoint( sun_x - 1, sun_y - 1 );
   p.drawPoint( sun_x + 1, sun_y - 1 );
   p.drawPoint( sun_x - 1, sun_y + 1 );
   p.drawPoint( sun_x + 1, sun_y + 1);
   
   p.end();
   worldMap->setBackgroundPixmap( tmpMap );
}

double PetitTrack::arccos( double x, double y )
{

   /* This function implements the arccosine function,
      returning a value between 0 and two pi.
      The code is borrowed from Predict.
    */

   double result = 0, fraction;

   fraction=x/y;

   if ((x>0.0) && (y>0.0))
      result=acos(fraction);

   if ((x<0.0) && (y>0.0))
      result=acos(fraction);

   if ((x<0.0) && (y<0.0))
      result = 3.14159 +  acos(fraction);

   if ((x>0.0) && (y<0.0))
      result = 3.14159 + acos(fraction);

   return result;
}

void PetitTrack::setRT1Object( int id )
{
   rt1ObjectID = id;
   calculateGroundTrack();
}

double PetitTrack::incrementDaynum( double daynum, double el )
{
   if      ( el >  45.0 ) daynum += 120.0 / 86400.0;
   else if ( el >  15.0 ) daynum +=  90.0 / 86400.0;
   else if ( el >   5.0 ) daynum +=  30.0 / 86400.0;
   else if ( el >   0.3 ) daynum +=  15.0 / 86400.0;
   else if ( el >  -0.3 ) daynum +=   0.8 / 86400.0;
   else if ( el >  -5.0 ) daynum +=  10.0 / 86400.0;
   else if ( el > -15.0 ) daynum +=  60.0 / 86400.0;
   else if ( el > -30.0 ) daynum += 180.0 / 86400.0;
   else if ( el > -45.0 ) daynum += 360.0 / 86400.0;
   else                   daynum += 720.0 / 86400.0;

   return daynum;
}

void PetitTrack::computeEphem()
{
   char name[81], line1[81], line2[81], temp[256];
   struct timeb tptr;

   // If we are already at work, don't bother.
   if ( calculating ) return;
   calculating = true;
   ephemStopFlag = false;
   
   strncpy( name, p[rt1ObjectID]->getSatName(), 80 );
   strncpy( line1, p[rt1ObjectID]->getTLELine1(), 80 );
   strncpy( line2, p[rt1ObjectID]->getTLELine2(), 80 );

   ephem = new KepPropagator();
   ephem->init( name, line1, line2, lat, lon, alt );

   ftime( &tptr );
   ephemDaynum = ((((double)tptr.time + 0.001 * (double)tptr.millitm) / 86400.0 ) - 3651.0);

   if ( !ephem->AosHappens() )
   {
      ephemView->clear();
      sprintf( temp, "There will be no passes for\n%s\n", ephem->getSatName() );
      ephemView->append( temp );
      sprintf( temp, "at your geographical coordinate.\n" );
      ephemView->append( temp );
      calculating = false;
      return;
   }
   
   ephemView->clear();
   sprintf( temp, "Ephemerides for %s\n", ephem->getSatName() );
   ephemView->append( temp );
   sprintf( temp, "From  %s\n", ephem->daynum2String( ephemDaynum ) );
   ephemView->append( temp );
   sprintf( temp, "---- AOS ---- -> ---- LOS ----" );
   ephemView->append( temp );

   ephemaos = 0;
   ephemlos = 0;
   ephemstart = ephemDaynum;
   ephemday = 0.0;
   epheminview = false;

   ephemTimer->start( 0, false );
}

void PetitTrack::runEphem()
{
   char aos_buf[40], los_buf[40], temp[80];
   time_t aos_t, los_t;
   struct tm *aos_tm, *los_tm;
   static int count = 0;

   while ( count++ < 10 )
   {
      ephem->propagate( ephemDaynum );
      ephemday = ephemDaynum - ephemstart;

      if ( !epheminview && ephem->getElevation() >= 0.0 )
      {
	 epheminview = true;
	 ephemaos = ephemDaynum;
      }
      if ( epheminview && ephem->getElevation() < 0.0 )
      {
	 epheminview = false;
	 ephemlos = ephemDaynum;
	 
	 aos_t = ephem->daynum2unix( ephemaos );
	 aos_tm = gmtime( &aos_t );
	 sprintf( aos_buf, "%02d%02d %02d:%02d:%02d",
		  aos_tm->tm_mon + 1, aos_tm->tm_mday, 
		  aos_tm->tm_hour, aos_tm->tm_min, aos_tm->tm_sec );
	 
	 los_t = ephem->daynum2unix( ephemlos );
	 los_tm = gmtime( &los_t );
	 sprintf( los_buf, "%02d%02d %02d:%02d:%02d",
		  los_tm->tm_mon + 1, los_tm->tm_mday, 
		  los_tm->tm_hour, los_tm->tm_min, los_tm->tm_sec );
	 sprintf( temp, "%s -> %s", aos_buf, los_buf );
	 ephemView->append( temp );
	 ephemDaynum += 55.5E-3;
	 ephem->propagate( ephemDaynum );
      }

      ephemDaynum = incrementDaynum( ephemDaynum, ephem->getElevation() );
      if ((int)ephemday > ephemDays || ephemStopFlag )
      {
	 ephemTimer->stop();
	 calculating = false;
	 ephemStopFlag = false;
	 delete ephem;
	 break;
      }
   }
   count = 0;
}

void PetitTrack::itemSelected( QListViewItem *listItem )
{
   char buf[3];
   int index;
   sprintf( buf, "%s", listItem->text( 4 ).latin1() );
   index = atoi( buf );
   setRT1Object( index );
}

void PetitTrack::calculateGroundTrack()
{
   KepPropagator *gtp;
   char name[81], line1[81], line2[81];
   struct timeb tptr;
   int t;
   double groundTrackDaynum;
   int x, y;
   
   strncpy( name, p[rt1ObjectID]->getSatName(), 80 );
   strncpy( line1, p[rt1ObjectID]->getTLELine1(), 80 );
   strncpy( line2, p[rt1ObjectID]->getTLELine2(), 80 );
   
   gtp = new KepPropagator();
   gtp->init( name, line1, line2, lat, lon, alt );

   ftime( &tptr );
   t = tptr.time / 120;
   t = t * 120;

   groundTrackDaynum = ((((double)t + 0.001 * (double)tptr.millitm) / 86400.0 ) - 3651.0);
   groundTrackDaynum -= 41.44447E-3; //Subtract 60 minutes
   
   for ( int i = 0; i < 60; i++ )
   {
      gtp->propagate( groundTrackDaynum );
      groundTrackDaynum += 1.38889E-3; // Increment 2 minutes
      if ( (int)gtp->getSspLon() <= 180 ) 
	 x = (int)((float)worldPix->height() - (float)worldPix->width() * gtp->getSspLon() / 360.0 );
      else 
	 x = (int)((float)worldPix->width() - (float)worldPix->width() * ( gtp->getSspLon() - 180.0 ) / 360.0 );
      
      y = (int)( (float)worldPix->height() / 2.0 - (float)worldPix->height() / 2.0 * gtp->getSspLat() / 90.0 );
      groundTrackPoints->setPoint( i, x, y );      
   }
   groundTrackFlag = true;
   delete gtp;
}

void PetitTrack::calculateSunFootprint()
{
   int azi, xx, yy;
   float ssplat, ssplong, azimuth;
   float num, dem, beta, spiderlat, spiderlong;
   float height = (float)worldPix->height();
   float width = (float)worldPix->width();
   float TWOPI = 2.0 * M_PI;
   float deg2rad = 1.74532925199e-02, R0=6378.16;
   struct timeb tptr;
   double daynum;
   
   ftime( &tptr );
   daynum = ((((double)tptr.time + 0.001 *
		      (double)tptr.millitm) / 86400.0 ) - 3651.0);
   
   sun->propagate( daynum );
   moon->propagate( daynum );
   ssplat = sun->getLat() * deg2rad;
   ssplong = sun->getLon() * deg2rad;
   beta = 0.5 * 20000 / R0;

   /* Compute the visibility circle */
   for ( azi = 0; azi < 360; azi++)
   {
      azimuth = deg2rad * (double)azi;
      spiderlat = asin( sin( ssplat ) * cos( beta )
			+ cos( azimuth ) * sin( beta ) * cos( ssplat ));
      num = cos( beta ) - ( sin( ssplat ) * sin( spiderlat ));
      dem = cos( ssplat ) * cos( spiderlat );

      if ( azi == 0 && ( beta > M_PI_2 - ssplat ))
	 spiderlong = ssplong + M_PI;
      else if ( azi == 180 && ( beta > M_PI_2 + ssplat ))
	 spiderlong = ssplong + M_PI;
      else if ( fabs( num / dem) > 1.0 )
	 spiderlong = ssplong;
      else
      {
	 if (( 180 - azi ) >= 0)
	    spiderlong = ssplong - arccos( num, dem );
	 else
	    spiderlong = ssplong + arccos( num, dem );
      }

      while ( spiderlong < 0.0 )
	 spiderlong += TWOPI;

      while ( spiderlong > ( 2.0 * M_PI ))
	 spiderlong -= TWOPI;

      spiderlat = spiderlat / deg2rad;
      spiderlong = spiderlong / deg2rad;

      /* Convert spiderweb data to map-based coordinates */
      if ( spiderlong <= 180.0 )
	 xx = (int)( height - ( width * ( spiderlong / 360.0 )));
      else
      {
	 spiderlong = spiderlong - 180.0;
	 xx = (int)( width - ( width * ( spiderlong / 360.0 )));
      }

      yy = (int)( height / 2.0 - ( height / 2.0 * ( spiderlat / 90.0 )));
      sunPoints->setPoint( azi, xx, yy );
   }   

   // The sun map coordinates 
   if ( sun->getLon() <= 180.0) sun_x = (int)( height - (width * ( sun->getLon() / 360.0 )));
   else sun_x = (int)( width - ( width * ( (sun->getLon() - 180.0) / 360.0 )));

   sun_y = (int)( height / 2.0 - ( height / 2.0 * ( sun->getLat() / 90.0 )));

   // The moon map corrdinates
   if ( moon->getLon() <= 180.0) moon_x = (int)( height - (width * ( moon->getLon() / 360.0 )));
   else moon_x = (int)( width - ( width * ( (moon->getLon() - 180.0) / 360.0 )));

   moon_y = (int)( height / 2.0 - ( height / 2.0 * ( moon->getLat() / 90.0 )));

}

void PetitTrack::setEphemDays( int days )
{
   ephemDays = days;
}

void PetitTrack::stopEphem()
{
   ephemStopFlag = true;
}

void PetitTrack::setMapType( int type )
{
   switch( type )
   {
   case 0:
      mapType = 0;
      worldPix = map0;
      break;
   case 1:
      mapType = 1;
      worldPix = map1;
      break;
   }
}
