#ifndef PETITTRACK_H
#define PETITTRACK_H

#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h> 
#include <sys/timeb.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include <qvariant.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qtabwidget.h>
#include <qlistview.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qmultilineedit.h>
#include <qbuttongroup.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qpe/config.h>

#include "keppropagator.h"
#include "sunpropagator.h"
#include "moonpropagator.h"

#define MAXOBJECTS 100

class PetitTrack : public QWidget
{ 
    Q_OBJECT

public:
   PetitTrack( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
   ~PetitTrack();
   
protected:
    QVBoxLayout* layout;
   
private:
   Config *MyConfiguration;
   QListView *rtnView;
   QListViewItem *listViewItem[ MAXOBJECTS ];
   QTabWidget *tab;
   QLabel* TextLabel1;
   QPushButton* quit;
   QLineEdit *lonLine, *latLine, *altLine, *tleLine;
   QFrame *rt1Frame;
   QFrame *ephemFrame;
   QFrame *worldMap;
   QPushButton *rt1Button[ MAXOBJECTS ];
   QMultiLineEdit *ephemView;
   QButtonGroup *rt1ButtonGroup;
   QPixmap *worldPix, *map0, *map1;
   QTimer *ephemTimer;
   QTimer *t1, *t2;
   QLabel *rtLabel[12];
   QLabel *aoslosLabel;
   QPointArray *groundTrackPoints;
   QPointArray *sunPoints;
   QSpinBox *ephemDaysSpin;
   QLabel *sunAziLabel, *sunEleLabel, *sunLatLabel, *sunLonLabel;
   QLabel *moonAziLabel, *moonEleLabel, *moonLatLabel, *moonLonLabel;
   QLabel *initLabel;
   QButtonGroup *mapButtonGroup;
   
   KepPropagator *p[ MAXOBJECTS ];
   KepPropagator *ephem;
   SunPropagator *sun;
   MoonPropagator *moon;
   
   int numObjects;
   double lat, lon;
   int alt;
   char tleFile[80];
   int rt1ObjectID;
   double aosTime[ MAXOBJECTS ];
   double losTime[ MAXOBJECTS ];
   bool calculating;
   double ephemDaynum;
   double ephemaos, ephemlos;
   double ephemstart;
   double ephemday;
   bool epheminview;
   bool groundTrackFlag;
   bool ephemStopFlag;
   int ephemDays;
   int sun_x, sun_y, moon_x, moon_y;
   int mapType;
   
   void loadTLE();
   void findAOSLOS( double, KepPropagator *, int );
   double incrementDaynum( double, double );
   void drawMap( KepPropagator * );
   double arccos( double, double );
   void calculateGroundTrack();
   void calculateSunFootprint();
   
public slots:
   void propagate();
   void setRT1Object( int );
   void computeEphem();
   void itemSelected( QListViewItem * );
   void runEphem();
   void setEphemDays( int );
   void stopEphem();
   void setMapType( int );
};

#endif
