/* Copyright (C) 2002 Edson Pereira, PU1JTE, N1VTN, 7N4NCL
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, download a copy
 * from http://www.fsf.org
 *
 *
 * Propagator algorithm derived from John A. Magliacane,
 * KD2BD's Predict.
 *
 */

#ifndef SUNPROPAGATOR_H
#define SUNPROPAGATOR_H

#include <math.h>

class SunPropagator
{
public:
   SunPropagator( double stnlat, double stnlon );
   ~SunPropagator();
  
   void propagate( double Daynum );
   double getAz();
   double getEl();
   double getLat();
   double getLon();
   
private:
   double c, ms, t1, t2, t3, ew, e, n, si, co, ra, gh,
          lh, gm, de, se, gs, dc, el[2], az[2], lat[2],
          lon[2], horizon_angle, yr, ssplat, ssplon;           
   double stnlat, stnlon;
   double daynum;
   double riseTime, setTime;
   
   static const double C1 = 0.91746406;
   static const double S1 = 0.397818675;
   static const double TP = 6.283185307179586;
   static const double deg2rad = 1.74532925199e-02;
 
   double fixAngle( double x );
};
#endif
